clc; clear; close all;
MaxIter=100;

load USPSTrain
numTrain=size(Xtrain,1); dim=size(Xtrain,2); 

Classes = CheckLabelVector(Ytrain);
numClasses = length(Classes);

% one - versus - all
for i=1:length(Classes)
  disp(['Class: ',num2str(Classes(i)),' versus all other ones']);
  LabelVec = 2*(Ytrain==Classes(i))-1;
  [W{i},aparam{i},Offset{i},cparam{i}]=GentleBoost(Xtrain,LabelVec,MaxIter);
end

load USPSTest
numTest = length(Ytest);
ResultTrain = zeros(numTrain,numClasses); ResultTest=zeros(numTest,numClasses);

% compute training and test error
for i=1:MaxIter
 for j=1:numClasses
   ResultTrain(:,j) = ResultTrain(:,j) + (aparam{j}(i)*(double(Xtrain*W{j}(:,i)+Offset{j}(i))>0)+cparam{j}(i));
   ResultTest(:,j)  = ResultTest(:,j)  + (aparam{j}(i)*(double(Xtest*W{j}(:,i)+Offset{j}(i))>0)+cparam{j}(i));
 end
 [Max,PredTrain] = max(ResultTrain');
 TrainError(i)   = sum(Classes(PredTrain)~=Ytrain);
 [Max,PredTest]  = max(ResultTest');
 TestError(i)    = sum(Classes(PredTest)~=Ytest);
 disp(['----------------------------------------------------------------------------------------------------']);
 disp(['Iteration: ',num2str(i),': TrainingErrors: ',num2str(TrainError(i)),' - %: ',num2str(TrainError(i)/numTrain),': TestErrors: ',num2str(TestError(i)),' - %: ',num2str(TestError(i)/numTest)]);
end

save('USPSResults.mat', 'ResultTrain', 'ResultTest', 'PredTrain', 'PredTest', 'TrainError', 'TestError');

figure,
hold on, 
plot(TrainError/numTrain,'k','LineWidth',3); plot(TestError/numTest,'r','LineWidth',3); 
xlabel('Iterations','FontSize',15); 
h=legend('Training error','Test error'); set(h,'FontSize',15); 
title('Training and test error on USPS of GentleBoost with stumps','FontSize',15); 
hold off

print('PlotTrainTestError.png', '-dpng');

Index=find(Classes(PredTest)~=Ytest);
Index=Index(1:min(end,49));
handles=VecToImage(Xtest(Index,:),16,16,0,2,1);
for i=1:length(Index)
  titleString=['True:',num2str(Ytest(Index(i))),' - Pred:',num2str(Classes(PredTest(Index(i))))];
  subplot(handles(i)); title(titleString,'FontWeight','bold');
end
